
#include <err.h>
#include <errno.h>
#include <fcntl.h>
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <sysexits.h>
#include <unistd.h>

#include <sys/types.h>
#include <sys/ioctl.h>

#include <dev/smbus/smb.h>

static const char *dev = "/dev/smb0";

static int fd;
static int slave = -1;
static int wflag;

static int
do_dump(void)
{
	int cmd;
	struct smbcmd c;
	unsigned char ibuf[SMB_MAXBLOCKSIZE];

	c.slave = slave;

	for (cmd = 0; cmd < 256; ++cmd) {
		c.data.byte_ptr = ibuf;
		c.cmd = cmd;
		if (ioctl(fd, wflag ? SMB_READW : SMB_READB, &c) == -1)
			return (-1);
		printf("%.2x: 0x%x\n", cmd, (int) (char) ibuf[0]);
	}
	return (0);
}

static void
usage(void)
{
	fprintf(stderr,
	    "usage: smbdump [-f dev] -s slave [-w]\n");
	exit(EX_USAGE);
}

int
main(int argc, char **argv)
{
	int i;

	while ((i = getopt(argc, argv, "f:s:w")) != -1)
		switch (i) {
		case 'f':
			dev = optarg;
			break;
		case 's':
			slave = strtoul(optarg, NULL, 0);
			break;
		case 'w':
			wflag = 1;
			break;
		default:
			usage();
		}

	if (slave == -1)
		usage();

	if ((fd = open(dev, O_RDWR)) == -1)
		err(EX_UNAVAILABLE, "Cannot open %s", dev);

	i = do_dump();
	if (i)
		errx(EX_UNAVAILABLE, "Error performing SMBus IO");

	return (0);
}
